@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2012 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2013 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -407,6 +407,7 @@ class Config:
     self.options.generate_etags = 1
     self.options.svn_ignore_mimetype = 0
     self.options.svn_config_dir = None
+    self.options.max_filesize_kbytes = 512
     self.options.use_rcsparse = 0
     self.options.sort_by = 'file'
     self.options.sort_group_dirs = 1
@@ -429,6 +430,7 @@ class Config:
     self.options.show_log_in_markup = 1
     self.options.cross_copies = 1
     self.options.use_localtime = 0
+    self.options.iso8601_timestamps = 0
     self.options.short_log_len = 80
     self.options.enable_syntax_coloration = 1
     self.options.tabsize = 8
